package prolog;

import prolog.util.Vector;

/**
 * <TT>';'/2</TT>: Represents a disjunctive goal in the body of a clause.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Disjunction.java,v 1.10 2000/10/02 12:09:33 peteg Exp $
 */
public class Disjunction extends Goal {
    public static final String FUNCTOR = ";".intern();
    public static final int    ARITY   = 2;

    Goal either_goal, or_goal;

    public Disjunction(Goal either_goal, Goal or_goal) {
	this.either_goal = either_goal;
	this.or_goal     = or_goal;
    }

    public boolean unify(Term[] insts, Prolog p)
	throws PrologException {

	System.err.println("Disjunction.unify(): pending_goal: " + either_goal);
	p.set_pending_goal(either_goal);
	p.allocate_cp(null, p.get_trail_top());
	p.use_pending_goal();

	return true;
    }

    public boolean redo(Term[] insts, Prolog p, ChoicePoint cp)
	throws PrologException {

	System.err.println("Disjunction.redo(): pending_goal: " + either_goal);
	p.set_pending_goal(or_goal);
	p.use_pending_goal();

	return true;
    }

    /**
     * Instantiate a list of Goals with the given substitutions. We
     * overwrite the old <TT>Compound</TT> objects in the list with their
     * instantiated counterparts.
     */
    void instantiate(Term[] insts) {
	either_goal.instantiate(insts);
	// FIXME
	or_goal.instantiate(insts);
    }

    Goal normalise(Variable[] vars,
		   int next_new_var,
		   Vector new_vars,
		   TypeDictionary types) {
	throw new RuntimeException("Disjunction.normalise()");
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("( ");
	out.append(either_goal.toString());
	out.append("\n    ;\n\t");
	out.append(or_goal.toString());
	out.append(" )");

	return out.toString();
    }
}
