package prolog;

import java.io.*;

import prolog.util.*;

/**
 * Accepts jobs and executes them. Used to support a separate thread for the
 * compiler, if it's worthwhile.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: JITThread.java,v 1.4 2000/09/28 12:28:13 peteg Exp $
 * @see prolog.PredicateCodeGen
 */
public final class JITThread implements QueueConsumer, Runnable, Serializable {
    /**
     * The job queue.
     */
    private Queue queue;

    JITThread() {
    }

    JITThread(Queue q) {
	queue = q;
    }

    /**
     * If we're not threaded, compile objects when they're handed to us,
     * blocking the interpreter while doing so.
     */
    public void consume(Object o) {
// 	System.err.println("JITThread.consume() -- got job.");
	((PredicateCodeGen)o).run();
    }

    /**
     * If we're threaded, wait for a job to do, then do it.
     */
    public void run() {
	while(true) {
// 	    System.err.println("JITThread.run() -- waiting for job.");
	    PredicateCodeGen job = (PredicateCodeGen)queue.dequeue();
// 	    System.err.println("JITThread.run() -- got job.");
	    job.run();
	}
    }
}
