package prolog;

import java.io.Serializable;

import prolog.util.Hashtable;
import prolog.util.Stack;

/**
 * The dictionary of types that a predicate has been invoked with.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: PredicateTypeDictionary.java,v 1.24 2000/11/12 12:22:45 peteg Exp $
 * @see prolog.JITPredicate
 * @see prolog.PredicateTypeRec
 */
class PredicateTypeDictionary implements Serializable {
    PredicateTypeRec ptrs;

    /**
     * Trail of type var instantiations.
     */
    private TypeTrail trail;

    PredicateTypeDictionary() {
	trail = new TypeTrail();
    }

    Predicate called(JITPredicate predicate, Term[] args, Prolog p) {
//    	System.err.println("PredTypeDict.called():");

	if(args != null) {
	    TermInfo[] arg_tis = new TermInfo[args.length];

	    // Get the types of the args.
	    for(int i = 0; i < args.length; i++) {
		Term arg = args[i] = args[i].getBinding();

		arg.typify(p.types, true);
		arg_tis[i] = arg.ti;
//  		System.err.println("\t" + arg + " -> " + arg_tis[i]);
	    }

	    // Backtracking search for this type.
	    for(PredicateTypeRec cur = ptrs; cur != null; cur = cur.next) {
		if(cur.unify(arg_tis, trail)) {
		    if(cur.compiled == null) {
			// Success. Bump the counter, think about compiling...
			if(++cur.called == p.threshold) {
			    p.db.compilerQueue.enqueue(new PredicateCodeGen(p,
									    predicate,
									    cur));
			}

			trail.unwind();
		    }

		    return cur.compiled;
		} else {
// 		    System.err.println("  PTD: failed.");
		    trail.unwind();
		}
	    }

	    // No match - create a new record.
 	    ptrs = new PredicateTypeRec(arg_tis, ptrs);
	}

	return null;
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("% -- PredicateTypeDictionary --\n");

	for(PredicateTypeRec ptr = ptrs; ptr != null; ptr = ptr.next) {
	    out.append(ptr);
	}

	return out.toString();
    }
}
