package prolog;

import java.io.Serializable;

import prolog.util.Hashtable;

/**
 * A node in the list of types that a <TT>JITPredicate</TT> has been invoked
 * with.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: PredicateTypeRec.java,v 1.7 2000/11/12 12:22:45 peteg Exp $
 * @see prolog.PredicateTypeDictionary
 */
class PredicateTypeRec implements Serializable {
    TermInfo[] arg_tis;
    int called;

    CompiledPredicate compiled;

    PredicateTypeRec next;

    PredicateTypeRec(TermInfo[] arg_tis, PredicateTypeRec next) {
	this.arg_tis = arg_tis;
	this.next    = next;

	called = 1;
    }

    boolean unify(TermInfo[] arg_tis, TypeTrail trail) {
	for(int i = 0; i < arg_tis.length; i++) {
	    if(!this.arg_tis[i].unify(arg_tis[i], trail)) {
// 		System.err.println("  PTR: " + this + " >>> arg" + i + " didn't match.");
		return false;
	    }
	}

	return true;
    }

    /**
     * Invoked by the compiler when the predicate has been compiled.
     */
    void setCompiled(CompiledPredicate p) {
	compiled = p;
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("  ");

	for(int i = 0; i < arg_tis.length; i++) {
	    out.append(arg_tis[i]);
	    out.append("\n");
	}

	out.append(" (");
	out.append(called);
	out.append(")");

	if(compiled == null) {
	    out.append(" (not compiled)\n");
	} else {
	    out.append(" (compiled)\n");
	}

	return out.toString();
    }
}
