package prolog.lib;

import prolog.*;
import prolog.math.Numeric;

/**
 * <TT>dynamic/1</TT>: Flag a predicate as being dynamic.
 * (assert-/retract-able).
 * <P>
 * <TT>dynamic(PredicateName/Arity)</TT>
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: dynamic_1.java,v 1.5 2000/08/10 12:51:03 peteg Exp $
 * @see prolog.DynamicPredicate
 * @see prolog.lib.abolish
 * @see prolog.lib.asserta_1
 * @see prolog.lib.assertz_1
 * @see prolog.lib.retract_1
 */
public class dynamic_1 extends CompiledPredicate {
    public dynamic_1() {
    }

    public boolean exec(Term[] args, Prolog p) throws PrologException {
	try {
	    Compound arg0 = (Compound)args[0];
 	    String f = ((Compound)arg0.args[0]).functor;
 	    int a = ((Numeric)arg0.args[1]).intValue();
	    Database db = p.db;

	    if(db.get(f, a) != null) {
                throw new PrologException("dynamic/1: predicate is already defined: " + arg0);
            }

            db.put(f, a, new DynamicPredicate(a));

	    return true;
	} catch (ClassCastException e) {
	    throw new ArgumentException("dynamic/1: invalid argument: ", args);
	}
    }
}
