package prolog.lib;

import prolog.*;
import prolog.math.Numeric;
import prolog.math.IntNum;

/**
 * <TT>functor/3</TT>: Build and pull apart <TT>Compound</TT> terms.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: functor_3.java,v 1.1 2000/09/15 15:11:35 peteg Exp $
 */
public class functor_3 extends CompiledPredicate {
    public functor_3() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
	    Term arg0 = args[0].getBinding();
	    Term arg1 = args[1].getBinding();
	    Term arg2 = args[2].getBinding();

	    if(arg0 instanceof Variable) {
		String functor = ((Compound)arg1).functor;
		int arity = ((Numeric)arg2).intValue();

		Term[] new_args = new Term[arity];

		for(int i = 0; i < arity; i++) {
		    new_args[i] = new Binding(i);
		}

		return arg0.unify(new Compound(functor, new_args), null, p);
	    } else {
		Compound c = (Compound)arg0;

		return arg1.unify(new Compound(c.functor), null, p)
		    && arg2.unify(IntNum.make(c.arity), null, p);
	    }
	} catch (ClassCastException e) {
	    throw new ArgumentException("functor/3: invalid arguments: ", args);
	}
    }

    public String toString() {
	return "functor/3";
    }
}
