package prolog.lib;

import prolog.*;
import prolog.math.Numeric;

/**
 * <TT>is_not_equal/2</TT>: The first argument evaluates to a number
 * distinct from the second (when both are interpreted as arithmetical
 * expressions).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: is_not_equal_2.java,v 1.9 2000/11/13 11:55:04 peteg Exp $
 * @see prolog.lib.is_2
 * @see prolog.lib.is_equal_2
 */
public class is_not_equal_2 extends CompiledPredicate {
    public static final String functor = "is_not_equal".intern();

    public is_not_equal_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
	    Term arg0 = args[0].getBinding(), arg1 = args[1].getBinding();
	    Numeric r0, r1;

//   	    System.err.println("is_not_equal/2: " + arg0 + ", " + arg1);

	    if(arg0 instanceof Compound) {
		r0 = is_2.eval((Compound)arg0);
	    } else {
		r0 = (Numeric)arg0;
	    }

	    if(arg1 instanceof Compound) {
		r1 = is_2.eval((Compound)arg1);
	    } else {
		r1 = (Numeric)arg1;
	    }

// 	    System.err.println("is_not_equal/2: r0: " + r0 + " r1: " + r1);

	    return !r0.equals(r1);
	} catch(ClassCastException e) {
	    throw new ArgumentException("is_not_equal/2: invalid arguments: ", args);
	}
    }

    public boolean compile_call(ClauseCodeGen ccg, Term[] args, boolean lastcall) {
	return ccg.is_not_eq_compile(args[0], args[1]);
    }
}
