package prolog.lib;

import prolog.*;
import prolog.math.Numeric;

/**
 * <TT>lt/2</TT>: Less than.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: lt_2.java,v 1.11 2000/11/12 12:22:45 peteg Exp $
 */
public class lt_2 extends CompiledPredicate {
    public static final String functor = "lt".intern();

    public lt_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
	    Term arg0 = args[0].getBinding(), arg1 = args[1].getBinding();
	    Numeric r0, r1;

	    if(arg0 instanceof Compound) {
		r0 = is_2.eval((Compound)arg0);
	    } else {
		r0 = (Numeric)arg0;
	    }

	    if(arg1 instanceof Compound) {
		r1 = is_2.eval((Compound)arg1);
	    } else {
		r1 = (Numeric)arg1;
	    }

	    return r1.compare(r0) == 1;

//   	    System.err.println("lt: " + arg0.val.toString() + " < " +  arg1.val.toString());
	} catch(ClassCastException e) {
	    throw new ArgumentException("lt/2: (<) invalid arguments: ", args);
	}
    }

    public boolean compile_call(ClauseCodeGen ccg, Term[] args, boolean lastcall) {
	return ccg.numeric_compare(args[1].ti, args[0].ti, 1);
    }
}
