package prolog.util;

/**
 * Asychronous, blocking queue (thread safe).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: ASyncQueue.java,v 1.3 2000/08/06 13:43:44 peteg Exp $
 */
public class ASyncQueue implements Queue {
    private Element head, tail;

    public ASyncQueue() {
	head = tail = null;
    }

    public synchronized void enqueue(Object o) {
	Element qe = new Element(o);

	if(head == null) {
	    head = tail = qe;
	    notify();
	} else {
	    head.next = qe;
	    head = qe;
	}
    }

    /**
     * Dequeue the next object, if there is one. Otherwise block until there
     * is one.
     */
    public synchronized Object dequeue() {
	if(tail == null) {
	    try {
		wait();
	    } catch(InterruptedException ie) {
		System.err.println("AsyncQueue: InterruptedException()");
	    }
	}

	Object o = tail.element;
	tail = tail.next;

	if(tail == null) {
	    head = null;
	}

	return o;
    }

    public boolean isEmpty() {
	return tail == null;
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	for(Element qe = tail; qe != null; qe = qe.next) {
	    out.append(qe.toString());
	}

	return out.toString();
    }
}
