package prolog;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

/**
 * Represents Lists, which are really <TT>Compound</TT> terms.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: ListType.java,v 1.1 2000/08/06 13:43:46 peteg Exp $
 */
class ListType extends CompoundType {
    ListType(TermType car, TermType cdr) {
	super(new TermType[]{car, cdr});
    }

    protected ListType() {
	super(new TermType[0]);
    }

    public boolean equals(Object o) {
	if(!(o instanceof ListType))
	    return(false);

	return(super.equals(o));
    }

    boolean unifiable(TermType t) {
	return(t == EmptyListType.type ||
	       (t instanceof ListType && super.unifiable(t)));
    }

    boolean compile_unify(TermInfo ti0, Term arg0, Term arg1, InstructionStrand is) {
	// If the other term has type Variable, let it do the work
	TermInfo ti1 = is.st.getInfo(arg1);

	if(ti1.type instanceof VariableType) {
	    throw new RuntimeException("ListType.compile(): other arg is a variable.");
	}

	if(!unifiable(ti1.type)) {
	    System.err.println("ListType.compile(): incompatible types: " + this + ", " + ti1.type);
	    return(false);
	}

	System.err.println("ListType.compile_unify(): " + arg0 + " " + this +
			   ", " + arg1 + " " + ti1.type);

	if(ti1.type == EmptyListType.type) {
	    System.err.println("ListType.compile_unify(): eol case.");
	    ti0.load(null, is);
	    ti1.load(arg1, is);
	    is.addFP(is.if_acmpne());
	    return(true);
	} else {
	    System.err.println("ListType.compile_unify(): general case.");

	    Term arg00 = null, arg01 = null, arg10 = null, arg11 = null;

	    if(arg0 instanceof Compound) {
		Compound c = (Compound)arg0;

		arg00 = c.args[0];
		arg01 = c.args[1];
	    }

	    if(arg1 instanceof Compound) {
		Compound c = (Compound)arg1;

		arg10 = c.args[0];
		arg11 = c.args[1];
	    }

 	    is.compound_loadargs(ti0, (Compound)(arg0 != null && arg0 instanceof Compound ?
						 arg0 : null));

	    return(args[0].compile_unify(arg00, arg10, is) &&
		   args[1].compile_unify(arg01, arg11, is));
	}
    }

    public TermType squash(TermType car) {
	return(car.equals(args[0].type) ? args[1].type : this);
    }

    public String toString() {
	StringBuffer sb = new StringBuffer();

	sb.append("L(");
	sb.append(args[0].type);
	sb.append(", ");
	sb.append(args[1].type);
	sb.append(")");
	return(sb.toString());
    }
}
