% -*- Prolog -*-
%
% Meta interpreter using rule/2.
%  from "The Art of Prolog" -- Leon Sterling, Ehud Shapiro, MIT Press.
%

solve(Goal) :- solve(Goal, []).

solve([], []).
solve([], [G|Goals]) :-	solve(G, Goals).
solve([A|B], Goals) :- append(B, Goals, Goals1), solve(A, Goals1).
solve(A, Goals) :- rule(A, B), solve(B, Goals).

% append/3 - concat two lists in order
append([], X, X).
append([X|Xs], Y, [X|Z]) :- append(Xs, Y, Z).

% nrev/2 - naive reverse
nrev([],[]).
nrev([X|Xs], Zs) :- nrev(Xs, Ys), append(Ys, [X], Zs).

% quicksort/3
qsort([X|L], R, R0) :-
	partition(L, X, L1, L2),
	qsort(L2, R1, R0),
	qsort(L1, R, [X|R1]).
qsort([], R, R).

partition([X|L], Y, [X|L1], L2) :-
	le(X, Y),
	partition(L, Y, L1, L2).
partition([X|L], Y, L1, [X|L2]) :-
	gt(X, Y),
	partition(L, Y, L1, L2).
partition([], X, [], []).

main :-
	solve(qsort([27, 74, 17, 33, 94, 18, 46, 83, 65, 2, 32, 53, 28, 85, 99,
	47, 28, 82, 6, 11, 55, 29, 39, 81, 90, 37, 10, 0, 66, 51, 7, 21, 85,
	27, 31,	63, 75, 4, 95, 99, 11, 28, 61, 74, 18, 92, 40, 53, 59, 8], X, [])).
