% -*- Prolog -*-
%
% Tracer using rule/2.
%  from "The Art of Prolog" -- Leon Sterling, Ehud Shapiro, MIT Press.
% (modified by peteg@cse.unsw.edu.au to use rule/2)

solve_trace(Goal) :-
	solve_trace(Goal, 0).

solve_trace([], Depth) :- !.
solve_trace([A|B], Depth) :-
	!, solve_trace(A, Depth), solve_trace(B, Depth).
solve_trace(A, Depth) :-
	builtin(A), !, call(A), display(A, Depth), nl.
solve_trace(A, Depth) :-
	rule(A, B), display(A, Depth), nl, is(Depth1, Depth + 1),
	solve_trace(B, Depth1).

display(A, Depth) :-
	is(Spacing, 3*Depth), put_spaces(Spacing), write(A).

put_spaces(N) :-
	between(1, N, I), write(" "), fail.
put_spaces(N).

between(I, J, I) :-
	le(I, J).
between(I, J, K) :-
	lt(I, J), is(I1, I + 1), between(I1, J, K).






% append/3 - concat two lists in order
append([], X, X).
append([X|Xs], Y, [X|Z]) :- append(Xs, Y, Z).

% nrev/2 - naive reverse
nrev([],[]).
nrev([X|Xs], Zs) :- nrev(Xs, Ys), append(Ys, [X], Zs).

% quicksort/3
qsort([X|L], R, R0) :-
	partition(L, X, L1, L2),
	qsort(L2, R1, R0),
	qsort(L1, R, [X|R1]).
qsort([], R, R).

partition([X|L], Y, [X|L1], L2) :-
	le(X, Y),
	partition(L, Y, L1, L2).
partition([X|L], Y, L1, [X|L2]) :-
	gt(X, Y),
	partition(L, Y, L1, L2).
partition([], X, [], []).

main :-
	solve_trace(qsort([2, 1, 3], X, [])).
%	solve_trace(nrev([1, 2, 3], X)).
