% -*- Prolog -*-

% append/3 - concat two lists in order
append([], X, X).
append([X|Xs], Y, [X|Z]) :- append(Xs, Y, Z).

% nrev/2 - naive reverse
nrev([],[]).
nrev([X|Xs],Zs) :- nrev(Xs,Ys), append(Ys,[X],Zs).

% rev/2 - more sophisticated reverse using an accumulator
rev(X, Y) :- rev(X, [], Y).

rev([], X, X).
rev([X|Xs], T, Y) :- rev(Xs, [X|T], Y).

% true/0 - always succeeds
true.

% isclosedlist/1 - succeeds iff the parameter is a properly formed list
isclosedlist([]).
isclosedlist([X|Xs]) :- isclosedlist(Xs).

% member/2 - determine if arg1 in arg2 (a list)
member(X, [X|Y]).
member(X, [Z|Y]) :- member(X,Y).

% nrev 30 has 496 logical inferences
nrev30 :- nrev([1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0], P).

% equals/2 - equality
equals(A, A).

main :- nrev30.
